import netgen.gui
from math import pi as myPi
from ngsolve import *
from netgen.csg import unit_cube

mesh = Mesh(unit_cube.GenerateMesh(maxh=0.1))
# for i in range(2):
#     mesh.Refine()

V = VectorH1(mesh, order=2, dirichlet="back|front")
print(mesh.GetBoundaries())
u, v = V.TnT()


# material parameters and forces
E, nu = 10.0, 0.3
mu = E / 2./(1+nu)
lmbda = 2 * mu * nu/(1-2*nu)
bodyForce = CoefficientFunction((0., -0.5, 0.))


# Define strain measures
I = Id(mesh.dim)
F = I + grad(u)
# help(F)
I1 = Trace(F.trans * F)
J = Det(F)
psi = mu/2. * (I1 - 3.) - mu * log(J) + lmbda/2. * (J - 1.0)**2
# S = mu * F - mu * (F.trans).Inv + lmbda * J * (J - 1.0) * (F.trans).Inv
factor=Parameter(1000.0)
# definition of bilinear and linear forms
a = BilinearForm(V, symmetric=True)
a += Variation(psi.Compile() * dx)
a += Variation((-1 * InnerProduct(bodyForce, u)).Compile() * dx)
# a += Variation((0.5/factor * InnerProduct(grad(u), grad(u))).Compile() * dx)

u = GridFunction(V)
u.vec[:] = 0.



# definition of the boundaries 
scale = 0.5
yo, zo = 0.5, 0.5
thta = myPi/3.

uLeft = CoefficientFunction((0.0, 0., 0.))
uRight = CoefficientFunction((
    0.,
    scale * (yo + (y - yo) * cos(thta) - (z - zo) * sin(thta) - y),
    scale * (zo + (y - yo) * sin(thta) + (z - zo) * cos(thta) - z)
))

u.Set(x*uRight)

Draw(u, mesh, "displacement", deformation=True)
SetVisualization(deformation=True)
#input("dirichlet data")
res = u.vec.CreateVector()
# resNew = u.vec.CreateVector()
w = u.vec.CreateVector()

maxiters = 50

"""
with TaskManager():
    a.Apply(u.vec, res)
    a.AssembleLinearization(u.vec)
    inv = a.mat.Inverse(V.FreeDofs(), inverse="sparsecholesky")
w.data = inv * res
resNorm = sqrt(abs(InnerProduct(w, w)))
u.vec.data -= w
Redraw()
input("B")
"""
resNorm = 1
iter = 0
with TaskManager():
    solvers.Newton(a, u, dampfactor=0.25, maxit=maxiters, maxerr=1.e-8, printing=True)
    """
    while resNorm > 1.e-8 and iter < maxiters:
        factor.Set(1.e6)
        a.Apply(u.vec, res)
        a.AssembleLinearization(u.vec)
        
        inv = a.mat.Inverse(V.FreeDofs(), inverse="sparsecholesky")
        w.data = inv * res
        resNorm = sqrt(abs(InnerProduct(w, w)))
        print(f"Norm: {resNorm}")
        u.vec.data -= w
        iter += 1
        Redraw()
        input("A")
    """
